<script type="application/ld+json">
{
  "@context": "https://schema.org",
  "@type": "SoftwareApplication",
  "name": "n8n-deploy",
  "applicationCategory": "DeveloperApplication",
  "operatingSystem": "Linux, macOS, Windows",
  "description": "{{ site.description }}",
  "url": "{{ site.url }}{{ site.baseurl }}",
  "author": {
    "@type": "Organization",
    "name": "{{ site.author.name }}",
    "url": "{{ site.author.url }}"
  },
  "offers": {
    "@type": "Offer",
    "price": "0",
    "priceCurrency": "USD"
  },
  "softwareVersion": "0.1.5",
  "datePublished": "2024-01-01",
  "programmingLanguage": "Python",
  "codeRepository": "https://github.com/lehcode/n8n-deploy",
  "license": "https://opensource.org/licenses/MIT",
  "keywords": "n8n, workflow, automation, CLI, Python, deployment, database, SQLite"
}
</script>

<script type="application/ld+json">
{
  "@context": "https://schema.org",
  "@type": "TechArticle",
  "headline": "{{ page.title }}",
  "description": "{{ page.description | default: site.description }}",
  "author": {
    "@type": "Organization",
    "name": "{{ site.author.name }}"
  },
  "publisher": {
    "@type": "Organization",
    "name": "n8n-deploy",
    "logo": {
      "@type": "ImageObject",
      "url": "{{ site.url }}{{ site.baseurl }}/assets/images/logo.png"
    }
  },
  "datePublished": "{{ page.date | default: site.time | date_to_xmlschema }}",
  "dateModified": "{{ page.last_modified_at | default: page.date | default: site.time | date_to_xmlschema }}"
}
</script>
